//--------------------------------------------------------------
//  残響生成器で使う遅延器
//
//  2023/06/13, Copyright (c) 2023 MIKAMI, Naoki
//--------------------------------------------------------------

#include "Array.hpp"

#ifndef RING_BUFFERE_HPP
#define RING_BUFFERE_HPP

class DelayUnit
{
public:
    // コンストラクタ
    explicit DelayUnit(int delay)
        : DELAY_(delay), dn_(delay, 0.0f), index_(0) {}

    // 遅延器のクリア
    void Clear() { dn_.Fill(0); }

    // データ取り出し
    operator float() const { return dn_[index_]; }

    // データ格納とインデックスのインクリメント
    void operator=(float x)
    {
        dn_[index_] = x;
        if (++index_ >=  DELAY_) index_ = 0;
    }

private:
    const int DELAY_;           // 遅延器の数
    Mikami::Array<float> dn_;   // 遅延素子に対応する配列
    int index_;
};    
#endif  // RING_BUFFERE_HPP
