//----------------------------------------------------------------------
//  残響生成器
//
//  2023/06/20, Copyright (c) 2023 MIKAMI, Naoki
//----------------------------------------------------------------------

#include "MultirateLiPh.hpp"
#include "Comb.hpp"
#include "Allpass.hpp"
#pragma diag_suppress 870   // マルチバイト文字使用の警告抑制のため
using namespace Mikami;

const float FS_ = 44.1f;    // 入力の標本化周波数： 44.1 kHz
MultirateLiPh myAdDa_(FS_); // 出力標本化周波数を４倍にするオブジェクト

const float G_C_ = 0.8f;
const float G_A_ = 0.6f;
const float G0_  = 1.0f - G_C_;
Comb    cm1(1997, G_C_);
Comb    cm2(2803, G_C_);
Comb    cm3(3559, G_C_);
Allpass ap1(  97, G_A_);
Allpass ap2( 131, G_A_);

bool on_ = true;

void AdcIsr()
{
    float xn = G0_*myAdDa_.Input(); // 入力

    // 櫛形フィルタの並列接続部
    float yn = cm1.Execute(xn) + cm2.Execute(xn) + cm3.Execute(xn);
    // 全域通過フィルタの縦続接続部
    yn = ap2.Execute(ap1.Execute(yn));

    yn = yn + xn;                   // 直接入力を付加

    if (on_) myAdDa_.Output(yn);    // 出力（残響生成器の出力）
    else     myAdDa_.Output(3*xn);  // 出力（入力をそのまま）
}

int main()
{
    setbuf(stdout, NULL);   // printf() でバッファ無しの設定
    printf("\r\n残響生成器を実行します\r\n");
    printf("残響生成器の ON/OFF はキーボードからコントロールできます\r\n");
    printf("'y'：ON, 'n'：OFF, ただし 'y' 以外は OFF になります\r\n");
    printf("起動時は ON の状態です\r\n? ");

    myAdDa_.Start(&AdcIsr);     // 入力の標本化開始
    while (true)
    {
        char yesNo = getchar();
        printf("%c\r\n? ", yesNo);
        on_ = (yesNo == 'y') ? true : false;
    }
}
