//---------------------------------------------------
//  縦続形 IIR フィルタ
//
//  2023/06/11, Copyright (c) 2023 MIKAMI, Naoki
//---------------------------------------------------

#include "Biquad.hpp"
#include "Array.hpp"    // Array クラスが定義されている
using Mikami::Array;

#ifndef IIR_CASCADE_HPP
#define IIR_CASCADE_HPP

class IirCascade
{
public:
    // コンストラクタ
    IirCascade(int order, const Biquad ck[], float g0)
        : ORDER2_((order+1)/2), G0_(g0),
          hn_(ORDER2_, ck) {}

    // フィルタ処理を実行する
    float Execute(float xn)
    {
        float yn = G0_*xn;
        for (int k=0; k<ORDER2_; k++) yn = hn_[k].Execute(yn);

        return yn;
    }

private:
    const int ORDER2_;  // 次数の半分（(order+1)/2）
    const float G0_;    // 利得定数
    Array<Biquad> hn_;  // Biquad クラスのオブジェクトの配列

    // コピー･コンストラクタ，代入演算子の禁止のため
    IirCascade(const IirCascade&);
    IirCascade& operator=(const IirCascade&);
};
#endif  // IIR_CASCADE_HPP
